#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# Dieses Dokument ist in der Zusammenarbeit von Informatikreferendaren und
# ehemaligen Informatikreferendaren der Studienseminare (heute ZfsL) Arnsberg,
# Hamm und Solingen entstanden.
#
# Das Dokument steht unter der Lizenz: Creative Commons by-nc-sa Version 4.0
# http://creativecommons.org/licenses/by-nc-sa/4.0/deed.de
#
# Nach dieser Lizenz darf das Dokument beliebig kopiert und bearbeitet werden,
# sofern das Folgeprodukt wiederum unter gleichen Lizenzbedingungen vertrieben
# und auf die ursprünglichen Urheber verwiesen wird.
# Eine kommerzielle Nutzung ist ausdrücklich ausgeschlossen.
#
# Die Namensnennung durch einen Verweis und die Lizenzangabe der ursprünglichen
# Urheber auf den Materialien für Schülerinnen und Schüler ist erforderlich.
#
# Die Sammlung der Dokumente steht unter
# http://ddi.uni-wuppertal.de/material/materialsammlung/
# zur Verfügung.
#
# Weitere Lizenz: GPL V3.0
# Siehe http://www.gnu.org/licenses/gpl-3.0.html
#
# $Id: basketball_demo3.py 1214 2015-05-12 15:21:38Z humbert $
#
# Gedacht u. a. als Rahmenprogramm zur Implementierung von Sequenzdiagrammen


def sortIn(listeEinAus):
    """Sortierung nach zwei Kriterien: Anzahl Siege und folgend Anzahl Punkte"""
    def key1(te): return -te.siege
    def key2(te): return -te.punktErzielt
    listeEinAus.sort(key=key2) 
    listeEinAus.sort(key=key1) 

    
class BasketballTurnier:
    def __init__(self, termin, aZeit, eZeit, ausrichter):
        self.datum= termin
        self.beginn= aZeit
        self.ende= eZeit
        self.veranstalter= ausrichter
        self.spiele = None
        self.tabelle = None
    def gibDatum(self):
        return self.datum
    def gibBeginn(self):
        return self.beginn
    def gibEnde(self):
        return self.ende
    def gibVeranstalter(self):
        return self.veranstalter
    def gibTabelle(self):
        return self.tabelle
    
    def spieleEintragen(self, *paarungen):
        self.spiele= paarungen
    def mannschaftenEintragen(self, *teams):
        self.mannschaften= teams
        self.tabelle= Tabelle(teams)
    def spieleAusgeben(self):
        for einzelspiel in self.spiele:
            print(einzelspiel.gibInfo())
    def gibSieger(self):
        if self.gibTabelle() == None:
            print("Es gibt noch keinen Sieger.")
            return None
        else:
            return self.tabelle.gibTabellenFuehrer()
    def gibSpielMitNummer(self, nr):
        if self.spiele == None:
            print("Dem Turnier wurden noch keine Spiele zugewiesen!")
        else:
            return self.spiele[nr]
    

class TabellenEintrag:
    def __init__(self, team):
        self.mannschaft= team
        self.spiele= 0
        self.siege= 0
        self.punktDiff= 0
        self.punktErzielt= 0
    def aktualisieren(self, gewinnz, punkteUnterschied, erzieltePunkte):
        self.spiele= self.spiele + 1
        self.siege= self.siege + gewinnz
        self.punktDiff= self.punktDiff + punkteUnterschied
        self.punktErzielt= self.punktErzielt + erzieltePunkte
    def __str__(self):
        return self.mannschaft.gibName().ljust(20) + " Spiele: " + str(self.spiele) + ", Siege: " + str(self.siege).rjust(2) + ", Diff: " + str(self.punktDiff).rjust(3) + ", Punkte: " + str(self.punktErzielt).rjust(3)
    
class Tabelle:
    def __init__(self, mannschaften):
        self.eintraege= [] # leere Liste
        for jedeMannschaft in mannschaften:
            self.eintrag= TabellenEintrag(jedeMannschaft)
            self.eintraege.append(self.eintrag)
    def ausgabe(self):
        sortIn(self.eintraege)      
        for jedesElement in self.eintraege:
            print(str(jedesElement))

    def ergebnisEintragen(self, spiel):
        self.sieger= spiel.gibSieger()
        if self.sieger != None:
            self.verlierer= spiel.gibVerlierer()
            self.punktDiff= spiel.gibPunkteSieger() - spiel.gibPunkteVerlierer()
            for jedesElement in self.eintraege:
                if jedesElement.mannschaft == self.sieger:
                    jedesElement.aktualisieren(1, self.punktDiff, spiel.gibPunkteSieger())
                if jedesElement.mannschaft == self.verlierer:
                    jedesElement.aktualisieren(0, -self.punktDiff, spiel.gibPunkteVerlierer())
        else:
            for jedesElement in self.eintraege:
                if jedesElement.mannschaft == spiel.mannschaftA:
                    jedesElement.aktualisieren(0, 0, spiel.punkteA)
                if jedesElement.mannschaft == spiel.mannschaftB:
                    jedesElement.aktualisieren(0, 0, spiel.punkteB)
    def gibTabellenFuehrer(self):
        sortIn(self.eintraege)       
        return self.eintraege[0].mannschaft
    
class BasketballSpiel:
    def __init__(self, terminturnier, zeitbeginn, zeitende, lokation, teamA, teamB):
        self.turnier= terminturnier
        self.beginn= zeitbeginn
        self.ende= zeitende
        self.ort= lokation
        self.mannschaftA= teamA
        self.mannschaftB= teamB
        self.punkteA= -1
        self.punkteB= -1
    def gibBeginn(self):
        return self.beginn
    def gibEnde(self):
        return self.ende
    def gibOrt(self):
        return self.ort
    def gibMannschaftA(self):
        return self.mannschaftA
    def gibMannschaftB(self):
        return self.mannschaftB
    
    def ergebnisEintragen(self, punkteA, punkteB):
        self.punkteA= punkteA
        self.punkteB= punkteB
        self.turnier.gibTabelle().ergebnisEintragen(self)
    def gibSieger(self):
        if self.punkteA > self.punkteB:
            return self.mannschaftA
        elif self.punkteB > self.punkteA:
            return self.mannschaftB
        else:
            return None
    def gibVerlierer(self):
        if self.punkteA > self.punkteB:
            return self.mannschaftB
        elif self.punkteB > self.punkteA:
            return self.mannschaftA
        else:
            return None
    def gibPunkteSieger(self):
        if self.punkteA < 0:
            return "Spiel hat noch nicht stattgefunden."
        if self.punkteA > self.punkteB:
            return self.punkteA
        else:
            return self.punkteB
    def gibPunkteVerlierer(self):
        if self.punkteA < 0:
            return "Spiel hat noch nicht stattgefunden."
        if self.punkteA > self.punkteB:
            return self.punkteB
        else:
            return self.punkteA
    def gibErgebnisSieger(self):
        if self.punkteA < 0:
            return "Spiel hat noch nicht stattgefunden!"
        return str(self.gibPunkteSieger()) + ":" + str(self.gibPunkteVerlierer())
    def gibInfo(self):
        self.info = "Spiel zwischen '" + self.mannschaftA.name + "' und '" + self.mannschaftB.name + "' in " + self.ort.name + " (" + self.beginn + "-" + self.ende + "):\n"
        if self.punkteA < 0:
            self.info += "  Ergebnis steht noch nicht fest."
        else:
            if self.gibSieger() != None:
                self.info += "  Gewonnen haben '" + self.gibSieger().name + "' mit " + self.gibErgebnisSieger() + "."
            else:
                self.info += "  Spiel endete unentschieden " + str(self.punkteA) + ":" + str(self.punkteB) + "."
        return self.info
    
class Mannschaft:
    def __init__(self, ersterName, kursbezeichnung, farbe):
        self.name= ersterName
        self.kurs= kursbezeichnung
        self.trikotfarbe= farbe
    def gibName(self):
        return self.name
    def gibKurs(self):
        return self.kurs
    def gibTrikotfarbe(self):
        return self.trikotfarbe
    def setzeName(self, neuerName):
        self.name= neuerName
    def setzeTrikotfarbe(self, neueFarbe):
        self.trikotfarbe= neueFarbe
    
class Sporthalle:
    def __init__(self, name, standort):
        self.name = name
        self.standort = standort
    def gibName(self):
        return self.name
    def gibStandort(self):
        return self.standort


if __name__ == "__main__":
    dieKaengurus = Mannschaft("Die Kängurus", "11d2", "gruen")
    crazyFrogs = Mannschaft("Crazy Frogs", "11d1", "blau")
    dieUeberflieger = Mannschaft("Die Überflieger", "11d3", "gelb")
    dieNowitzkis = Mannschaft("Die Nowitzkis", "11d4", "grau")
    
    basketballTurnier = BasketballTurnier("16.08.2037", "9:30", "12:30", "Willy-Schule - Bergkamen")
    
    halleA = Sporthalle("Halle A", "Altbau")
    halleB = Sporthalle("Halle B", "Neubau")
    
    spielAB = BasketballSpiel(basketballTurnier, "09:45", "10:15", halleA, dieKaengurus, dieNowitzkis)
    spielCD = BasketballSpiel(basketballTurnier, "10:00", "10:30", halleB, dieUeberflieger, crazyFrogs)
    spielAC = BasketballSpiel(basketballTurnier, "10:45", "11:15", halleA, dieKaengurus, dieUeberflieger)
    spielBD = BasketballSpiel(basketballTurnier, "11:00", "11:30", halleB, dieNowitzkis, crazyFrogs)
    spielAD = BasketballSpiel(basketballTurnier, "11:45", "12:15", halleA, dieKaengurus, crazyFrogs)
    spielBC = BasketballSpiel(basketballTurnier, "12:00", "12:30", halleB, dieNowitzkis, dieUeberflieger)
    
    basketballTurnier.spieleEintragen(spielAB, spielCD, spielAC, spielBD, spielAD, spielBC)
    basketballTurnier.mannschaftenEintragen(dieKaengurus, crazyFrogs, dieUeberflieger, dieNowitzkis)
    
    basketballTurnier.spieleAusgeben()
    print ("-----")
    
    spielAB.ergebnisEintragen(32, 20)
    basketballTurnier.gibTabelle().ausgabe()
    print()    
    spielCD.ergebnisEintragen(24, 23)
    basketballTurnier.gibTabelle().ausgabe()
    print()    
    spielAC.ergebnisEintragen(31, 22)
    basketballTurnier.gibTabelle().ausgabe()
    print()    
    spielBD.ergebnisEintragen(15, 36) # Änderung gegenüber der Vorlage wg. Sort
    basketballTurnier.gibTabelle().ausgabe()
    print()    
    spielAD.ergebnisEintragen(26, 31)
    basketballTurnier.gibTabelle().ausgabe()
    print()    
    spielBC.ergebnisEintragen(27, 27)
    
    basketballTurnier.spieleAusgeben()
    print ("-----")
    
    print ("Gesamtsieger: ", basketballTurnier.gibSieger().gibName() )
    basketballTurnier.gibTabelle().ausgabe()
