#! /usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Dieses Dokument ist in der Zusammenarbeit von Informatikreferendaren und
# ehemaligen Informatikreferendaren der Studienseminare (heute ZfsL) Arnsberg,
# Hamm und Solingen entstanden.
#
# Das Dokument steht unter der Lizenz: Creative Commons by-nc-sa Version 4.0
# http://creativecommons.org/licenses/by-nc-sa/4.0/deed.de
#
# Nach dieser Lizenz darf das Dokument beliebig kopiert und bearbeitet werden,
# sofern das Folgeprodukt wiederum unter gleichen Lizenzbedingungen vertrieben
# und auf die ursprünglichen Urheber verwiesen wird.
# Eine kommerzielle Nutzung ist ausdrücklich ausgeschlossen.
#
# Die Namensnennung durch einen Verweis und die Lizenzangabe der ursprünglichen
# Urheber auf den Materialien für Schülerinnen und Schüler ist erforderlich.
#
# Die Sammlung der Dokumente steht unter
# http://ddi.uni-wuppertal.de/material/materialsammlung/
# zur Verfügung.
#
# Weitere Lizenz: GPL V3.0
# Siehe http://www.gnu.org/licenses/gpl-3.0.html
#
# $Id: spielturnier_small.py 1245 2015-06-03 15:26:49Z humbert $
#
# Implementierung einer Lösung für das Basketballturnier 
# in der Programmiersprache Python (Version 3.x).
# Kompatibilität mit Version 2.x
from __future__ import print_function
#
from turniertabelle_ds import SpielTabelle

class BasketballTurnier:
    def __init__(self, datum, beginn, ende, veranstalter):
        self.setzeDatum(datum)
        self.setzeBeginn(beginn)
        self.setzeEnde(ende)
        self.setzeVeranstalter(veranstalter)        
        #self.spiele= None
        #self.tabelleMannschaften= None        
        
    def setzeDatum(self, dat):
        self.datum= dat
    def gibDatum(self):
        return self.datum

    def setzeBeginn(self, anfzeit):
        self.beginn= anfzeit
    def gibBeginn(self):
        return self.beginn

# wg. Kompatibilität: Beginn => Start
    def setzeStart(self, anfzeit):
        self.beginn= anfzeit
    def gibStart(self):
        return self.beginn
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    def setzeEnde(self, endzeit):
        self.ende= endzeit
    def gibEnde(self):
        return self.ende

    def setzeVeranstalter(self, veranst):
        self.veranstalter= veranst
    def gibVeranstalter(self):
        return self.veranstalter

# wg. Kompatibilität: Veranstalter => Ausrichter
    def setzeAusrichter(self, veranst):
        self.veranstalter= veranst
    def gibAusrichter(self):
        return self.veranstalter
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



class BasketballSpiel:
    def __init__(self, turnier, beginn, ende, ort, mannschaftA, mannschaftB):
        self.setzeTurnier(turnier)
        self.setzeBeginn(beginn)
        self.setzeEnde(ende)
        self.setzeOrt(ort)
        self.setzeMannschaftA(mannschaftA)
        self.setzeMannschaftB(mannschaftB)
        self.setzePunkteA(-1)
        self.setzePunkteB(-1)
    
    def setzeTurnier(self, tur):
        self.turnier = tur

    def setzeBeginn(self, anf):
        self.beginn= anf
    def gibBeginn(self):
        return self.beginn

    def setzeEnde(self, end):
        self.ende= end
    def gibEnde(self):
        return self.ende

    def setzeOrt(self, wo):
        self.ort= wo
    def gibOrt(self):
        return self.ort

    def setzeMannschaftA(self, mA):
        self.mannschaftA= mA        
    def gibMannschaftA(self):
        return self.mannschaftA

    def setzeMannschaftB(self, mB):
        self.mannschaftB= mB
    def gibMannschaftB(self):
        return self.mannschaftB

    def setzePunkteA(self, pA):
        self.punkteA= pA
    def gibPunkteA(self):
        return self.punkteA
        
    def setzePunkteB(self, pB):
        self.punkteB= pB        
    def gibPunkteB(self):
        return self.punkteB

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# in den folgenden Kommentaren wird X verwendet
# mit X Element aus 1,2
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# wg. Kompatibilität: gibMannschaftX, setzeMannschaftX
    def setzeMannschaft1(self, mA):
        self.mannschaftA= mA        
    def gibMannschaft1(self):
        return self.mannschaftA

    def setzeMannschaft2(self, mB):
        self.mannschaftB= mB
    def gibMannschaft2(self):
        return self.mannschaftB

# wg. Kompatibilität: gibTeamX, setzeTeamX
    def setzeTeam1(self, mA):
        self.mannschaftA= mA        
    def gibTeam1(self):
        return self.mannschaftA

    def setzeTeam2(self, mB):
        self.mannschaftB= mB
    def gibTeam2(self):
        return self.mannschaftB

# wg. Kompatibilität: gibPunkteX, setzePunkteX
    def setzePunkte1(self, pA):
        self.punkteA= pA
    def gibPunkte1(self):
        return self.punkteA
        
    def setzePunkte2(self, pB):
        self.punkteB= pB        
    def gibPunkte2(self):
        return self.punkteB
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    def ergebnisEintragen(self, punkteA, punkteB):
        self.setzePunkteA(punkteA)
        self.setzePunkteB(punkteB)
        #self.turnier.gibTabelle().ergebnisEintragen(self)

    def ergebnisAnfordernUndEintragen(self):
        self.ergebnisEintragen(
         *map(int, input("'"+self.gibTeam1().gibName()+ "' gegen '"
         +self.gibTeam2().gibName()
         +"' – Spielergebnis als xx:yy ").split(":"))
        )

        
    def gibSieger(self):
        if self.punkteA > self.punkteB:
            return self.mannschaftA
        elif self.punkteB > self.punkteA:
            return self.mannschaftB
        else:
            return None
    
    def gibVerlierer(self):
        if self.punkteA > self.punkteB:
            return self.mannschaftB
        elif self.punkteB > self.punkteA:
            return self.mannschaftA
        else:
            return None
    
    def gibPunkteSieger(self):
        if self.punkteA < 0:
            return "Das Spiel hat noch nicht stattgefunden!"
        if self.punkteA > self.punkteB:
            return self.punkteA
        else:
            return self.punkteB
    
    def gibPunkteVerlierer(self):
        if self.punkteA < 0:
            return "Das Spiel hat noch nicht stattgefunden!"
        if self.punkteA > self.punkteB:
            return self.punkteB
        else:
            return self.punkteA
    
    def gibErgebnisSieger(self):
        if self.punkteA < 0:
            return "Das Spiel hat noch nicht stattgefunden!"
        return str(self.gibPunkteSieger()) + ":" + str(self.gibPunkteVerlierer())

    def gibSieger(self):
        if self.punkteA > self.punkteB:
            return self.mannschaftA
        elif self.punkteA < self.punkteB:
            return self.mannschaftB
        else:
            return None

    def gibVerlierer(self):
        if self.punkteA < self.punkteB:
            return self.mannschaftA
        elif self.punkteA > self.punkteB:
            return self.mannschaftB
        else:
            return None

    def gibPunkteSieger(self):
        if self.punkteA < 0:
            return "Das Spiel hat noch nicht stattgefunden!"
        if self.punkteA > self.punkteB:
            return self.punkteA
        else:
            return self.punkteB
    
    def gibPunkteVerlierer(self):
        if self.punkteA < 0:
            return "Das Spiel hat noch nicht stattgefunden!"
        if self.punkteA > self.punkteB:
            return self.punkteB
        else:
            return self.punkteA

    def gibErgebnisSieger(self):
        if self.punkteA < 0:
            return "Das Spiel hat noch nicht stattgefunden!"
        return str(self.gibPunkteSieger()) + ":" + str(self.gibPunkteVerlierer())

    def gibInfo(self):
        self.info= "Spiel zwischen '" + self.mannschaftA.name + "' und '" + self.mannschaftB.name + "' in " + self.ort.name + " (" + self.beginn + "-" + self.ende + ").\n"
        if self.punkteA < 0:
            self.info += "  Das Ergebnis steht noch nicht fest."
        else:
            if self.gibSieger() != None:
                self.info += "  Gewonnen haben '" + self.gibSieger().name + "' mit " + self.gibErgebnisSieger() + "."
            else:
                self.info += "  Das Spiel endete unentschieden " + str(self.punkteA) + ":" + str(self.punkteB) + "."
        return self.info


class Mannschaft:
    def __init__(self, name, kurs, trikotfarbe):
        self.setzeName(name)
        self.__setzeKurs(kurs)
        self.setzeTrikotfarbe(trikotfarbe)

    def setzeName(self, neuerName):
        self.name= neuerName
    def gibName(self):
        return self.name

    def __setzeKurs(self,ku):
        self.kurs= ku
    def gibKurs(self):
        return self.kurs

    def setzeTrikotfarbe(self, neueFarbe):
        self.trikotfarbe= neueFarbe
    def gibTrikotfarbe(self):
        return self.trikotfarbe

    
class Sporthalle:
    def __init__(self, name, standort):
        self.setzeName(name)
        self.__setzeStandort(standort)

    def setzeName(self, neuerName):
        self.name= neuerName
    def gibName(self):
        return self.name

    def __setzeStandort(self, neuerStandort):
        self.standort= neuerStandort
    def gibStandort(self):
        return self.standort


if __name__ == "__main__":
    basketballTurnier= BasketballTurnier(
      "16.08.2037", "9:30", "12:30", "Willy-Brandt-Gesamtschule Bergkamen"
      )

    dieKaengurus= Mannschaft("Die Kängurus", "11d2", "gruen")
    crazyFrogs= Mannschaft("Crazy Frogs", "11d1", "blau")
    dieUeberflieger= Mannschaft("Die Überflieger", "11d3", "gelb")
    dieNowitzkis= Mannschaft("Die Nowitzkis", "11d4", "grau")

    halleA= Sporthalle("Halle A", "Altbau")
    halleB= Sporthalle("Halle B", "Neubau")

    spielAB= BasketballSpiel(basketballTurnier, "09:45", "10:15", 
      halleA, dieKaengurus, dieNowitzkis)
    spielCD= BasketballSpiel(basketballTurnier, "10:00", "10:30", halleB,
      dieUeberflieger, crazyFrogs)
    spielAC= BasketballSpiel(basketballTurnier, "10:45", "11:15", halleA, 
      dieKaengurus, dieUeberflieger)
    spielBD= BasketballSpiel(basketballTurnier, "11:00", "11:30", halleB, 
      dieNowitzkis, crazyFrogs)
    spielAD= BasketballSpiel(basketballTurnier, "11:45", "12:15", halleA,  
      dieKaengurus, crazyFrogs)
    spielBC= BasketballSpiel(basketballTurnier, "12:00", "12:30", halleB, 
      dieNowitzkis, dieUeberflieger)
    
    
    spielTabelle=SpielTabelle(
      dieKaengurus, crazyFrogs, dieUeberflieger, dieNowitzkis)
    spielTabelle.ausgabe()

    print ("-----")    
    
    #spielAB.ergebnisAnfordernUndEintragen()
    spielAB.ergebnisEintragen(32, 20)
    spielTabelle.ergebnisEintragen(spielAB)
    spielTabelle.ausgabe()
    print()    
    #spielCD.ergebnisAnfordernUndEintragen()
    spielCD.ergebnisEintragen(24, 23)
    spielTabelle.ergebnisEintragen(spielCD)
    spielTabelle.ausgabe()
    print()    
    #spielAC.ergebnisAnfordernUndEintragen()
    spielAC.ergebnisEintragen(31, 22)
    spielTabelle.ergebnisEintragen(spielAC)
    spielTabelle.ausgabe()
    print()    
    #spielBD.ergebnisAnfordernUndEintragen()
    spielBD.ergebnisEintragen(15, 33)
    spielTabelle.ergebnisEintragen(spielBD)
    spielTabelle.ausgabe()
    print()    
    #spielAD.ergebnisAnfordernUndEintragen()
    spielAD.ergebnisEintragen(26, 31)
    spielTabelle.ergebnisEintragen(spielAD)
    spielTabelle.ausgabe()
    print()    
    #spielBC.ergebnisAnfordernUndEintragen()
    spielBC.ergebnisEintragen(27, 27)    
    spielTabelle.ergebnisEintragen(spielBC)
    spielTabelle.ausgabe()
    print ("-----")   
    
    print ("Gesamtsieger: ", spielTabelle.gibTabellenFuehrer().gibName())
    spielTabelle.ausgabe()

