#! /usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Dieses Dokument ist in der Zusammenarbeit von Informatikreferendaren und
# ehemaligen Informatikreferendaren der Studienseminare (heute ZfsL) Arnsberg,
# Hamm und Solingen entstanden.
#
# Das Dokument steht unter der Lizenz: Creative Commons by-nc-sa Version 4.0
# http://creativecommons.org/licenses/by-nc-sa/4.0/deed.de
#
# Nach dieser Lizenz darf das Dokument beliebig kopiert und bearbeitet werden,
# sofern das Folgeprodukt wiederum unter gleichen Lizenzbedingungen vertrieben
# und auf die ursprünglichen Urheber verwiesen wird.
# Eine kommerzielle Nutzung ist ausdrücklich ausgeschlossen.
#
# Die Namensnennung durch einen Verweis und die Lizenzangabe der ursprünglichen
# Urheber auf den Materialien für Schülerinnen und Schüler ist erforderlich.
#
# Die Sammlung der Dokumente steht unter
# http://ddi.uni-wuppertal.de/material/materialsammlung/
# zur Verfügung.
#
# Weitere Lizenz: GPL V3.0
# Siehe http://www.gnu.org/licenses/gpl-3.0.html
#
# $Id: turniertabelle.py 1246 2015-06-03 15:34:47Z humbert $
#
# Kompatibilität mit Python Version 2.x
from __future__ import print_function
#

from datenstrukturen import Liste 

def sortIn(listeEinAus):
    """Sortierung nach zwei Kriterien: Anzahl Siege und folgend Anzahl Punkte"""
    # Die Reihenfolge ist wichtig: zunächst alle Einträge nach der Anzahl der
    # erzielten Punkte ordnen und erst dann nach Anzahl der Siege 
    # umgekehrt wäre falsch
    def key1(te): return -te.siege
    def key2(te): return -te.punkteErzielt
    listeEinAus.sort(key=key2) 
    listeEinAus.sort(key=key1) 


class TabellenSortierer:
    __module__ = __name__

    def sortiere(self, eintraege): # eintraege: Objekt der Klasse Liste
        liste= list() # Objekt der eingebauten Standardklasse
        zaehler= 0
        while zaehler < eintraege.gibLaenge():
            liste.append(eintraege.gibElementAnPosition(zaehler))
            zaehler= zaehler + 1

        sortIn(self.eintraege)  # mit der oben angegebenen Sortierfunktion
        eintraege= Liste()
        for eintrag in liste:
            eintraege.haengeAn(eintrag)

        return eintraege


class SpielTabelle:
    def __init__(self, mannschaftsListe):
        self.eintraege= []  # leere Liste
        for mannschaft in mannschaftsListe:
            self.eintrag= TabellenEintrag(mannschaft)
            self.eintraege.append(self.eintrag)

    def ausgabe(self):
        sortIn(self.eintraege)
        for eintrag in self.eintraege:
            print (str(eintrag))

    def ergebnisEintragen(self, spiel):
        self.sieger= spiel.gibSieger()
        if self.sieger != None:
            self.verlierer= spiel.gibVerlierer()
            self.punktDiff= spiel.gibPunkteSieger() - spiel.gibPunkteVerlierer()
            for eintrag in self.eintraege:
                if eintrag.mannschaft == self.sieger:
                    eintrag.aktualisieren(1, self.punktDiff, spiel.gibPunkteSieger())
                if eintrag.mannschaft == self.verlierer:
                    eintrag.aktualisieren(0, -self.punktDiff, spiel.gibPunkteVerlierer())
        else:
            for eintrag in self.eintraege:
                if eintrag.mannschaft == spiel.mannschaftA:
                    eintrag.aktualisieren(0, 0, spiel.punkteA)
                if eintrag.mannschaft == spiel.mannschaftB:
                    eintrag.aktualisieren(0, 0, spiel.punkteB)

    def gibTabellenFuehrer(self):
        sortIn(self.eintraege)
        return self.eintraege[0].mannschaft


class TabellenEintrag:
    def __init__(self, mannschaft):
        self.setzeMannschaft(mannschaft)
        self.setzeSpiele(0)
        self.setzeSiege(0)
        self.setzePunktDiff(0)
        self.setzePunkteErzielt(0)
    
    def setzeMannschaft(self, mschaft):
        self.mannschaft= mschaft

    def setzeSpiele(self, zahl):
        self.spiele= zahl
    def gibSpiele(self):
        return self.spiele        
    
    def setzeSiege(self, zahl):
        self.siege= zahl 
    def gibSiege(self):
        return self.siege

    def setzePunktDiff(self, zahl):  
        self.punktDiff= zahl
    def gibPunktDiff(self):
        return self.punktDiff
    
    def setzePunkteErzielt(self, zahl):    
        self.punkteErzielt= zahl
    def gibPunkteErzielt(self):
        return self.punkteErzielt
        
    def aktualisieren(self, siege, punktDiff, punkteErzielt):
        self.setzeSpiele(self.gibSpiele() + 1)
        self.setzeSiege(self.gibSiege() + siege)
        self.setzePunktDiff(self.gibPunktDiff() + punktDiff)
        self.setzePunkteErzielt(self.gibPunkteErzielt() + punkteErzielt)
    
    def __str__(self):
        return self.mannschaft.gibName().ljust(12) + "\t Spiele: " + str(self.gibSpiele()) + ", Siege: " + str(self.gibSiege()).rjust(2) + ", Diff: " + str(self.gibPunktDiff()).rjust(3) + ", Punkte: " + str(self.gibPunkteErzielt()).rjust(3)



